{ Algorithm: Linear Equation: Crout LU Decomposition }

program CroutLUDecomposition(output);
const n = 4;
	m = 3;
type vector = array[1..n] of real;
	matrix = array[1..n] of vector;
	vectors = array[1..m] of vector;
var aa : matrix;
	xx, bb : vectors;
	i,j,l : integer;
	
procedure swap(var x, y : real);
var temp : real;
begin
	temp := x;
	x := y;
	y := temp;
end;

procedure solve(a : matrix; var x : vectors;  b : vectors);
var i,j,k,l,iPivot,jPivot : integer;
	pivot,r,s : real;
	y, balancer : vector;
	a1, a2 : matrix;

procedure checkPivot(x : real);
begin
	if 0.0 = x then writeLn('warning: The coefficient matrix is not full-rank.');
end;

procedure decompose(var a, b, c : matrix);
var i,j,k : integer;
	pivot,r : real;
begin
	{ Decompose a into b * c by Crout's algorithm }
	for i := 1 to n do begin
		b[i][1] := a[i][1];
		c[i][1] := 0.0;
	end;
	pivot := a[1][1];
	c[1][1] := 1.0;
	for j := 2 to n do begin
		b[1][j] := 0.0;
		c[1][j] := a[1][j] / pivot;
	end;
	for i := 2 to n do begin
		r := a[i][i];
		for k := 1 to i do r := r - b[i][k] * c[k][i];
		b[i][i] := r;
		c[i][i] := 1.0;
		pivot := r;
		checkPivot(pivot);
		for j := i+1 to n do begin
			r := a[j][i];
			for k := 1 to i-1 do r := r - b[j][k] * c[k][i];
			b[j][i] := r;
			c[j][i] := 0.0;
			r := a[i][j];
			for k := 1 to i-1 do r := r - b[i][k] * c[k][j];
			c[i][j] := r / pivot;
			b[i][j] := 0.0;
		end;
	end;

	{ print result }
	writeLn('LU decomposed:');
	for i := 1 to n do begin
		write('{');
		for j := 1 to n do write(a[i][j]:5:1);
		if i = n div 2 then write('} = {') else write('}   {');
		for j := 1 to n do write(b[i][j]:5:1);
		write('}{');
		for j := 1 to n do write(c[i][j]:5:1);
		writeLn('}');
	end;
	writeLn;
end;

{ solve }
begin
	{ balance the equation }
	for i := 1 to n do begin
		r := 0.0;
		for j := 1 to n do begin
			s := abs(a[i][j]);
			if r < s then r := s;
		end;
		if r = 0.0 then r := 1.0;
		for j := 1 to n do a[i][j] := a[i][j] / r;
		for l := 1 to m do b[l][i] := b[l][i] / r;
	end;
	for j := 1 to n do begin
		r := 0.0;
		for i := 1 to n do begin
			s := abs(a[i][j]);
			if r < s then r := s;
		end;
		if r = 0.0 then r := 1.0;
		for i := 1 to n do a[i][j] := a[i][j] / r;
		balancer[j] := r;
	end;
	
	{ LU decomposition }
	decompose(a,a1,a2);
	for l := 1 to m do begin
		y[1] := b[l][1] / a1[1][1];
		for i := 2 to n do begin
			r := b[l][i];
			for j := 1 to i-1 do r := r - a1[i][j] * y[j];
			y[i] := r / a1[i][i];
		end;
		x[l][n] := y[n] / a2[n][n];
		for i := n downto 1 do begin
			r := y[i];
			for j := i+1 to n do r := r - a2[i][j] * x[l][j];
			x[l][i] := r / a2[i][i];
		end;
	end;
	
	for l := 1 to m do begin
{		for k := n-1 downto 1 do swap(x[l][k], x[l][order[k]]);}
		for k := 1 to n do x[l][k] := x[l][k] / balancer[k];
	end;
end;

procedure printEquation(var a : matrix; var x,b : vectors);
var i,j,l : integer;
begin
	for i := 1 to n do begin
		write('{');
		for j := 1 to n do write(a[i][j]:6:1, ' ');
		write('}{');
		for l := 1 to m do write(x[l][i]:6:1, ' ');
		if i = n div 2 then write('} = {') else write('}   {');
		for l := 1 to m do write(b[l][i]:6:1, ' ');
		writeLn('}');
	end;
end;

function getA(i,j : integer) : real;
begin
	getA := round(random * 10);
end;

function getB(l,i : integer) : real;
begin
	getB := round(random * 10);
end;

{ Main }
begin
	for i := 1 to n do for j := 1 to n do aa[i][j] := getA(i,j);
	for l := 1 to m do for i := 1 to n do bb[l][i] := getB(l,i);
	solve(aa,xx,bb);
	printEquation(aa,xx,bb);
end.