(* case field *)
program casefield(input, output);
type
	nodes = ( n_value, n_unary, n_binary, n_parent );
	values = ( v_integer, v_real, v_bool );
	ops = ( o_plus, o_minus, o_multi, o_div, o_parent );
	pnode = ^node;
	node = record
		ordinal : integer;
		case nodetype : nodes of
			n_value : (
				input : boolean;
				case valuetype : values of
					v_integer : ( n : integer );
					v_real : ( r : real);
					v_bool : ( z : boolean)
				);
			n_unary, n_parent : (
				utype : ops;
				operand : pnode;
				);
			n_binary : (
				btype : ops;
				operand1 : pnode;
				operand2 : pnode;
				)
		end;
var root, a, b, c : pnode;

function new_const_integer(v : integer) : pnode;
var pn : pnode;
begin
	new(pn, n_value, v_integer);
	pn^.nodetype := n_value;
	pn^.input := true;
	pn^.valuetype := v_integer;
	pn^.n := v;
	new_const_integer := pn;
end;

function new_integer : pnode;
var pn : pnode;
begin
	new(pn, n_value, v_integer);
	pn^.nodetype := n_value;
	pn^.input := false;
	pn^.valuetype := v_integer;
	new_integer := pn;
end;

function new_binary(op : ops; left, right : pnode) : pnode;
var pn : pnode;
begin
	new(pn, n_binary);
	pn^.nodetype := n_binary;
	pn^.btype := op;
	pn^.operand1 := left;
	pn^.operand2 := right;
	new_binary := pn;
end;

function new_unary(op : ops; right : pnode) : pnode;
var pn : pnode;
begin
	new(pn, n_unary);
	pn^.nodetype := n_unary;
	pn^.utype := op;
	pn^.operand := right;
	new_unary := pn;
end;

function new_parent(content : pnode) : pnode;
var pn : pnode;
begin
	new(pn, n_parent);
	pn^.nodetype := n_parent;
	pn^.utype := o_parent;
	pn^.operand := content;
	new_parent := pn;
end;

procedure eval(top : pnode); forward;

procedure eval_value(top : pnode);
begin
	if top^.input then begin
		case top^.valuetype of
			v_integer : write(top^.n);
			v_real : write(top^.r);
			v_bool : if top^.z then write('true') else write('false')
		end
	end else begin
		case top^.valuetype of
			v_integer : readln(top^.n);
			v_real : readln(top^.r);
			v_bool : top^.z := false
		end
	end
end;

procedure eval_unary(top : pnode);
begin
	case top^.utype of
		o_plus : write('+');
		o_minus : write('-');
	end;
	eval(top^.operand);
end;

procedure eval_binary(top : pnode);
begin
	eval(top^.operand1);
	case top^.utype of
		o_plus : write('+');
		o_minus : write('-');
		o_multi : write('*');
		o_div : write('/');
	end;
	eval(top^.operand2);
end;

procedure eval_parent(top : pnode);
begin
	write('(');
	eval(top^.operand);
	write(')');
end;

procedure eval;
begin
	case top^.nodetype of
		n_value : eval_value(top);
		n_unary : eval_unary(top);
		n_binary : eval_binary(top);
		n_parent : eval_parent(top)
	end
end;

begin
	a := new_integer;
	b := new_const_integer(7);
	c := new_binary(o_plus, a, b);
	a := new_integer;
	b := new_parent(c);
	c := new_binary(o_multi, a, b);
	root := new_unary(o_minus, c);

	eval(root);

	new(a, n_value, v_real);
	dispose(a, n_value, v_real);
end.